/*
 * Copyright (c) 2021, MediaTek Inc. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef __MTK_APUSYS_APC_H__
#define __MTK_APUSYS_APC_H__

void set_apusys_apc(void);

#endif /* __MTK_APUSYS_APC_H__ */
