/*
 * Copyright (c) 2018-2020, Broadcom
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef BCM_CONSOLE_H
#define BCM_CONSOLE_H

void bcm_console_boot_init(void);
void bcm_console_boot_end(void);
void bcm_console_runtime_init(void);
void bcm_console_runtime_end(void);

#endif /* BCM_CONSOLE_H */
