/*
 * Copyright (c) 2021, Stephan Gerhold <stephan@gerhold.net>
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MSM8916_GICV2_H
#define MSM8916_GICV2_H

void msm8916_gicv2_init(void);

#endif /* MSM8916_GICV2_H */
