/*
 * Copyright (c) 2016, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PWM_H
#define PWM_H

void disable_pwms(void);
void enable_pwms(void);

#endif /* PWM_H */
