/*
 * Copyright (c) 2015, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MBEDTLS_COMMON_H
#define MBEDTLS_COMMON_H

void mbedtls_init(void);

#endif /* MBEDTLS_COMMON_H */
