/*
 * Copyright (c) 2016, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include "../bl2_private.h"

/*******************************************************************************
 * Place holder function to perform any Secure SVC specific architectural
 * setup. At the moment there is nothing to do.
 ******************************************************************************/
void bl2_arch_setup(void)
{

}
