/*
 * Copyright (c) 2016, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef SP_MIN_PRIVATE_H
#define SP_MIN_PRIVATE_H

void sp_min_main(void);
void sp_min_warm_boot(void);
void sp_min_fiq(void);

#endif /* SP_MIN_PRIVATE_H */
