/*
 * Copyright (C) 2018 Marvell International Ltd.
 *
 * SPDX-License-Identifier:     BSD-3-Clause
 * https://spdx.org/licenses
 */

#ifndef MVEBU_DEF_H
#define MVEBU_DEF_H

#include <a8k_plat_def.h>

#define CP_COUNT		2	/* A80x0 has both CP0 & CP1 */
#define I2C_SPD_ADDR		0x53	/* Access SPD data */
#define I2C_SPD_P0_ADDR		0x36	/* Select SPD data page 0 */

#endif /* MVEBU_DEF_H */
