/*
 * Copyright (c) 2015-2018, STMicroelectronics - All Rights Reserved
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef STM32MP1_RAM_H
#define STM32MP1_RAM_H

int stm32mp1_ddr_probe(void);

#endif /* STM32MP1_RAM_H */
