/*
 * Copyright (c) 2021 Sipeed
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef SUNXI_CCU_H
#define SUNXI_CCU_H

#define SUNXI_CCU_SEC_SWITCH_REG	(SUNXI_CCU_BASE + 0x0f00)

#define SUNXI_R_PRCM_SEC_SWITCH_REG	(SUNXI_R_PRCM_BASE + 0x0290)

#endif /* SUNXI_CCU_H */
