/*
 * Copyright (c) 2021, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef FVP_R_BL1_H
#define FVP_R_BL1_H

void bl1_load_bl33(void);
void bl1_transfer_bl33(void);

#endif /* FVP_R_BL1_H */
