/*
 * Copyright (C) 2018 Marvell International Ltd.
 *
 * SPDX-License-Identifier:     BSD-3-Clause
 * https://spdx.org/licenses
 */

#ifndef PLAT_MACROS_S
#define PLAT_MACROS_S

#include <marvell_macros.S>

/*
 * Required platform porting macros
 * (Provided by included headers)
 */
.macro plat_crash_print_regs
.endm

#endif /* PLAT_MACROS_S */
