/*
 * Copyright (c) 2015-2021, Renesas Electronics Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef RCAR_VERSION_H
#define RCAR_VERSION_H

#include <arch_helpers.h>

#define VERSION_OF_RENESAS		"3.0.3"
#define VERSION_OF_RENESAS_MAXLEN	128

extern const uint8_t version_of_renesas[VERSION_OF_RENESAS_MAXLEN];

#endif /* RCAR_VERSION_H */
