/*
 * Copyright (c) 2015-2018, Renesas Electronics Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef QOS_INIT_H
#define QOS_INIT_H

extern void rcar_qos_init(void);
extern uint8_t get_boardcnf_phyvalid(void);

#endif /* QOS_INIT_H */
