/*
 * Copyright (c) 2016-2019, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PLAT_PARAMS_H
#define PLAT_PARAMS_H

#include <stdint.h>

#include <export/plat/rockchip/common/plat_params_exp.h>

#endif /* PLAT_PARAMS_H */
