/*
 * Copyright (c) 2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef SGM_VARIANT_H
#define SGM_VARIANT_H

/* SSC_VERSION values for sgm */
#define SGM775_SSC_VER_PART_NUM		0x0790

/* DMC configuration for sgm */
#define SGM_DMC_SIZE			0x40000
#define SGM775_DMC_COUNT		4

#endif /* SGM_VARIANT_H */
