/*
 * Copyright (c) 2015, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef TBBR_IMG_DESC_H
#define TBBR_IMG_DESC_H

#include <common/bl_common.h>

extern image_desc_t bl1_tbbr_image_descs[];

#endif /* TBBR_IMG_DESC_H */
