/*
 * Copyright (c) 2019, STMicroelectronics - All Rights Reserved
 *
 * SPDX-License-Identifier: GPL-2.0+ OR BSD-3-Clause
 */

#ifndef STM32_QSPI_H
#define STM32_QSPI_H

int stm32_qspi_init(void);

#endif /* STM32_QSPI_H */
