/*
 * Copyright (C) 2021 Marvell International Ltd.
 *
 * SPDX-License-Identifier:     BSD-3-Clause
 * https://spdx.org/licenses
 */

#ifndef MVEBU_DEF_H
#define MVEBU_DEF_H

#include <a8k_plat_def.h>

#define CP_COUNT		1	/* A70x0 has single CP0 */

#endif /* MVEBU_DEF_H */
