/*
 * Copyright (c) 2015-2017, Renesas Electronics Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PFC_INIT_H3_V2_H
#define PFC_INIT_H3_V2_H

void pfc_init_h3_v2(void);

#endif /* PFC_INIT_H3_V2_H */
