/*
 * Copyright (c) 2017-2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef K3_CONSOLE_H
#define K3_CONSOLE_H

void bl31_console_setup(void);

#endif /* K3_CONSOLE_H */
