/*
 * Copyright (c) 2018-2019, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef STDDEF__H
#define STDDEF__H

#ifndef SIZET_
typedef unsigned long size_t;
#define SIZET_
#endif

#endif /* STDDEF__H */
