/*
 * Copyright (c) 2021, STMicroelectronics - All Rights Reserved
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef REGULATOR_FIXED_H
#define REGULATOR_FIXED_H

int fixed_regulator_register(void);

#endif /* REGULATOR_FIXED_H */
