/*
 * Copyright (c) 2020, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef SGI_PLAT_H
#define SGI_PLAT_H

/* BL31 platform setup common to all SGI based platforms */
void sgi_bl31_common_platform_setup(void);

#endif /* SGI_PLAT_H */
