/*
 * Copyright (c) 2015-2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef IMX8_IOMUX_H
#define IMX8_IOMUX_H

#define PADRING_IFMUX_EN_SHIFT		U(31)
#define PADRING_IFMUX_EN_MASK		(U(0x1) << PADRING_IFMUX_EN_SHIFT)
#define PADRING_GP_EN_SHIFT		U(30)
#define PADRING_GP_EN_MASK		(U(0x1) << PADRING_GP_EN_SHIFT)
#define PADRING_IFMUX_SHIFT		U(27)
#define PADRING_IFMUX_MASK		(U(0x7) << PADRING_IFMUX_SHIFT)
#define PADRING_CONFIG_SHIFT		U(25)
#define PADRING_CONFIG_MASK		(U(0x3) << PADRING_CONFIG_SHIFT)
#define PADRING_LPCONFIG_SHIFT		U(23)
#define PADRING_LPCONFIG_MASK		(U(0x3) << PADRING_LPCONFIG_SHIFT)
#define PADRING_PULL_SHIFT		U(5)
#define PADRING_PULL_MASK		(U(0x3) << PADRING_PULL_SHIFT)
#define PADRING_DSE_SHIFT		U(0)
#define PADRING_DSE_MASK		(U(0x7) << PADRING_DSE_SHIFT)

#endif /* IMX8_IOMUX_H */
