/*
 * Copyright (c) 2016, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef ADDRESSMAP_H
#define ADDRESSMAP_H

#include <addressmap_shared.h>

/* Registers base address for M0 */
#define MMIO_BASE			0x40000000

#endif /* ADDRESSMAP_H */
