/*
 * Copyright (c) 2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

/*
 * TZC programming is currently not done.
 */
void plat_arm_security_setup(void)
{
}
