/*
 * Copyright 2021 NXP
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef PLATFORM_DEF_H
#define PLATFORM_DEF_H

#include <plat_def.h>
#include <plat_default_def.h>

#endif /* PLATFORM_DEF_H */
