/*
 * Copyright (c) 2019, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef __PLAT_SIP_CALLS_H__
#define __PLAT_SIP_CALLS_H__

#define RK_PLAT_SIP_NUM_CALLS	0

#endif /* __PLAT_SIP_CALLS_H__ */
