/*
 * Copyright (c) 2020, MediaTek Inc. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MTK_DCM_H
#define MTK_DCM_H

#include <stdbool.h>

bool dcm_set_default(void);

#endif /* #ifndef MTK_DCM_H */
