/*
 * Copyright (c) since 2022, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef MT_SPM_EXTERN_H
#define MT_SPM_EXTERN_H

void spm_extern_initialize(void);

#endif
