/*
 * Copyright (c) 2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef IMX_GPT_H
#define IMX_GPT_H

#include <stdint.h>

void imx_gpt_ops_init(uintptr_t reg_base);

#endif /* IMX_GPT_H */
