/*
 * Copyright (c) 2019-2021, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef ARM_TRUSTED_FIRMWARE_EXPORT_DRIVERS_GPIO_EXP_H
#define ARM_TRUSTED_FIRMWARE_EXPORT_DRIVERS_GPIO_EXP_H

/* EXPORT HEADER -- See include/export/README for details! -- EXPORT HEADER */

#define ARM_TF_GPIO_DIR_OUT		0
#define ARM_TF_GPIO_DIR_IN		1

#define ARM_TF_GPIO_LEVEL_LOW		0
#define ARM_TF_GPIO_LEVEL_HIGH		1

#define ARM_TF_GPIO_PULL_NONE		0
#define ARM_TF_GPIO_PULL_UP		1
#define ARM_TF_GPIO_PULL_DOWN		2
#define ARM_TF_GPIO_PULL_REPEATER	3

#endif /* ARM_TRUSTED_FIRMWARE_EXPORT_DRIVERS_GPIO_EXP_H */
