/*
 * Copyright 2020-2021 NXP
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 */

#ifndef IFC_NOR_H
#define IFC_NOR_H


int ifc_nor_init(uintptr_t flash_addr, size_t flash_size);

#endif /*IFC_NOR_H*/
