/*
 * Copyright (c) 2017, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef HI3660_MEM_MAP_H
#define HI3660_MEM_MAP_H

#define HISI_DATA_HEAD_BASE		(0x89C44400)

#define HISI_RESERVED_MEM_BASE		(0x89C80000)
#define HISI_RESERVED_MEM_SIZE		(0x00040000)

#define HISI_DATA0_BASE			(0x89C96180)
#define HISI_DATA0_SIZE			(0x000003A0)
#define HISI_DATA1_BASE			(0x89C93480)
#define HISI_DATA1_SIZE			(0x00002D00)

#endif /* HI3660_MEM_MAP_H */
