/*
 * Copyright (c) 2018-2020, The Linux Foundation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#ifndef QTISECLIB_DEFS_PLAT_H
#define QTISECLIB_DEFS_PLAT_H

#define QTISECLIB_PLAT_CLUSTER_COUNT	1
#define QTISECLIB_PLAT_CORE_COUNT	8

#define BL31_BASE						0x80b00000
#define BL31_SIZE						0x00100000

/* Chipset specific secure interrupt number/ID defs. */
#define QTISECLIB_INT_ID_SEC_WDOG_BARK			(0x204)
#define QTISECLIB_INT_ID_NON_SEC_WDOG_BITE		(0x21)

#define QTISECLIB_INT_ID_VMIDMT_ERR_CLT_SEC		(0xE6)
#define QTISECLIB_INT_ID_VMIDMT_ERR_CLT_NONSEC		(0xE7)
#define QTISECLIB_INT_ID_VMIDMT_ERR_CFG_SEC		(0xE8)
#define QTISECLIB_INT_ID_VMIDMT_ERR_CFG_NONSEC		(0xE9)

#define QTISECLIB_INT_ID_XPU_SEC			(0xE3)
#define QTISECLIB_INT_ID_XPU_NON_SEC			(0xE4)

#define QTISECLIB_INT_ID_A2_NOC_ERROR			(0x194)
#define QTISECLIB_INT_ID_CONFIG_NOC_ERROR		(0xE2)
#define QTISECLIB_INT_ID_DC_NOC_ERROR			(0x122)
#define QTISECLIB_INT_ID_MEM_NOC_ERROR			(0x6C)
#define QTISECLIB_INT_ID_SYSTEM_NOC_ERROR		(0xC6)
#define QTISECLIB_INT_ID_MMSS_NOC_ERROR			(0xBA)

#endif /* QTISECLIB_DEFS_PLAT_H */
