/*
 * Copyright (C) 2018, ARM Limited and Contributors. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#ifndef IMX_HAB_ARCH_H
#define IMX_HAB_ARCH_H

/* Define the offset the High Assurance Boot callback table is at */
#define HAB_CALLBACK_OFFSET	0x100

#endif /* IMX_HAB_ARCH_H */
